local explosion_texture = Resources.load_texture("spell_explosion.png")
local fireball_texture = Resources.load_texture("fireball.png")
local sfx = Resources.load_audio("sounds/fireball.ogg")

function card_init(player, props)
	--Use frames with specific timing.
	local frame1 = { 1, 2 }
	local frame2 = { 2, 2 }
	local frame3 = { 3, 2 }
	local frame4 = { 4, 10 }
	--Override the shooting animation with 3 sets of buster attacks.
	local frame_sequence = { frame1, frame2, frame1, frame1, frame1, frame2, frame1, frame1, frame1, frame2, frame3,
		frame4 }
	local action = Action.new(player, "CHARACTER_SHOOT")
	action:override_animation_frames(frame_sequence)
	action:set_lockout(ActionLockout.new_animation())
	action.on_execute_func = function(self, user)
		local hit_props = HitProps.new(
			props.damage,
			props.hit_flags,
			props.element,
			user:context(),
			Drag.None
		)

		local buster = self:create_attachment("BUSTER")
		local buster_sprite = buster:sprite()
		buster_sprite:set_texture(user:texture())
		buster_sprite:set_layer(-2)
		buster_sprite:use_root_shader(true)

		local buster_anim = buster:animation()
		buster_anim:copy_from(user:animation())
		buster_anim:set_state("BUSTER")
		self:add_anim_action(2, function()
			local spell1 = create_fireball(player, hit_props)
			player:field():spawn(spell1, player:get_tile(player:facing(), 1))
		end)

		self:add_anim_action(6, function()
			local spell2 = create_fireball(player, hit_props)
			player:field():spawn(spell2, player:get_tile(player:facing(), 1))
		end)

		self:add_anim_action(10, function()
			props.hit_flags = props.hit_flags & Hit.Flash --Add flashing to the final hit
			local spell3 = create_fireball(player, hit_props)
			player:field():spawn(spell3, player:get_tile(player:facing(), 1))
		end)
	end
	return action
end

function create_fireball(player, props)
	-- Set the spell to use your team
	local spell = Spell.new(player:team())

	-- Use the facing to stop the spell being backwards if used at the edge of your area
	local direction = player:facing()
	spell:set_facing(direction)

	-- Make it look like a fireball
	spell:set_texture(fireball_texture)

	-- Set the hit properties
	spell:set_hit_props(props)

	-- Use layering to draw on top of enemies
	local sprite = spell:sprite()
	sprite:set_layer(-1)

	-- obtain and set up the animatino so it knows how to draw
	-- instead of just using the entire sprite sheet
	local animation = spell:animation()
	animation:load("fireball.animation")
	animation:set_state("DEFAULT")

	-- Not gonna lie, not sure why I did this but whatever.
	spell:set_height(50)

	-- Need the field to spawn stuff
	local field = player:field()

	-- Determine if we hit anything
	spell.has_hit = false
	spell.on_update_func = function(self)
		-- get the tile.
		-- do this sort of thing to avoid calling the same functions over and over.
		local spell_tile = spell:current_tile()
		-- deal damage.
		spell_tile:attack_entities(self)

		-- If the tile is an edge and we're moving, say goodbye
		if spell_tile:is_edge() and self.slide_started then
			self:delete()
		end

		-- Obtain the destination tile.
		local dest = self:get_tile(spell:facing(), 1)

		-- Refer to self within slide using a variable
		local ref = self

		-- Only slide if we haven't hit anything yet
		if not self.has_hit then
			--Slide to the next tile at 5 frames per tile. It's meant to be fast.
			self:slide(dest, 5, function() ref.slide_started = true end)
		end
	end

	spell.on_collision_func = function(self, other)
		self.has_hit = true

		-- Use a spell instead of an artifact so it doesn't animate during time freeze.
		-- If you want something to move during time freeze, use an artifact.
		local fx1 = Spell.new(self:team())

		-- Set up the texture for this too.
		fx1:set_texture(explosion_texture)

		-- Set up the animatino for this too.
		local fx1_anim = fx1:animation()
		fx1_anim:load("spell_explosion.animation")
		fx1_anim:set_state("Default")
		-- Set the layer lower than the fireball
		-- Appears on top of enemies AND spells using layer -1.
		fx1:sprite():set_layer(-2)
		fx1_anim:on_complete(function() fx1:erase() end)
		-- Spawn the explosion for colliding.
		field:spawn(fx1, self:current_tile())
		self:erase()
	end

	-- Just return true so that no panel blocks it. Let it hit an enemy.
	spell.can_move_to_func = function(tile) return true end
	-- Play the audio from the global scope
	Resources.play_audio(sfx)
	return spell
end
